Re: [PATCH 2/2] perf: arm_dsu: Support DSU ACPI devices.

From: Suzuki K Poulose
Date: Wed Mar 18 2020 - 20:40:59 EST


Hello,


Please find my comments below.

On 03/18/2020 12:28 AM, Tuan Phan wrote:
Add support for probing device from ACPI node.
Each DSU ACPI node defines "cpus" package which
each element is the MPIDR of associated cpu.

Signed-off-by: Tuan Phan <tuanphan@xxxxxxxxxxxxxxxxxxxxxx>
---
drivers/perf/arm_dsu_pmu.c | 53 +++++++++++++++++++++++++++++++++++++++-------
1 file changed, 45 insertions(+), 8 deletions(-)

diff --git a/drivers/perf/arm_dsu_pmu.c b/drivers/perf/arm_dsu_pmu.c Tua
index 2622900..6ef762c 100644
--- a/drivers/perf/arm_dsu_pmu.c
+++ b/drivers/perf/arm_dsu_pmu.c
@@ -11,6 +11,7 @@
#define DRVNAME PMUNAME "_pmu"
#define pr_fmt(fmt) DRVNAME ": " fmt
+#include <linux/acpi.h>
#include <linux/bitmap.h>
#include <linux/bitops.h>
#include <linux/bug.h>
@@ -603,18 +604,22 @@ static struct dsu_pmu *dsu_pmu_alloc(struct platform_device *pdev)
}
/**
* dsu_pmu_dt_get_cpus: Get the list of CPUs in the cluster.
*/
-static int dsu_pmu_dt_get_cpus(struct device_node *dev, cpumask_t *mask)
+static int dsu_pmu_get_cpus(struct platform_device *pdev)
{
+#ifndef CONFIG_ACPI
+ /* Get the list of CPUs from device tree */

What if we have a kernel with both:

CONFIG_OF=y
CONFIG_ACPI=y

and boot the kernel on a system with DT ? In other words, the decision
to choose the DT vs ACPI must be runtime decision, not buildtime.

See drivers/hwtracing/coresight/coresight-platform.c:coresight_get_platform_data() for an example.

int i = 0, n, cpu;
struct device_node *cpu_node;
+ struct dsu_pmu *dsu_pmu =
+ (struct dsu_pmu *) platform_get_drvdata(pdev);
- n = of_count_phandle_with_args(dev, "cpus", NULL);
+ n = of_count_phandle_with_args(pdev->dev.of_node, "cpus", NULL);
if (n <= 0)
return -ENODEV;
for (; i < n; i++) {
- cpu_node = of_parse_phandle(dev, "cpus", i);
+ cpu_node = of_parse_phandle(pdev->dev.of_node, "cpus", i);
if (!cpu_node)
break;
cpu = of_cpu_node_to_id(cpu_node);
@@ -626,9 +631,33 @@ static int dsu_pmu_dt_get_cpus(struct device_node *dev, cpumask_t *mask)
*/
if (cpu < 0)
continue;
- cpumask_set_cpu(cpu, mask);
+ cpumask_set_cpu(cpu, &dsu_pmu->associated_cpus);
}
return 0;
+#else /* CONFIG_ACPI */
+ int i, cpu, ret;
+ const union acpi_object *obj;
+ struct acpi_device *adev = ACPI_COMPANION(&pdev->dev);
+ struct dsu_pmu *dsu_pmu =
+ (struct dsu_pmu *) platform_get_drvdata(pdev);
+

+ ret = acpi_dev_get_property(adev, "cpus", ACPI_TYPE_ANY, &obj);

Is the binding documented somewhere ?


nit: Also, why not :
ret = acpi_dev_get_propert(adev, "cpus", ACPI_TYPE_PACKAGE, &obj);
if (ret < 0)
return ret;
?


+ if (ret < 0)
+ return -EINVAL;
+
+ if (obj->type != ACPI_TYPE_PACKAGE)
+ return -EINVAL;
+
+ for (i = 0; i < obj->package.count; i++) {


+ /* Each element is the MPIDR of associated cpu */
+ for_each_possible_cpu(cpu) {
+ if (cpu_physical_id(cpu) ==
+ obj->package.elements[i].integer.value)
+ cpumask_set_cpu(cpu, &dsu_pmu->associated_cpus);
+ }
+ }
+ return 0;
+#endif
}

Otherwise looks good to me.
@@ -603,18 +604,22 @@ static struct dsu_pmu *dsu_pmu_alloc(struct platform_device *pdev)
}
/**
- * dsu_pmu_dt_get_cpus: Get the list of CPUs in the cluster.
+ * dsu_pmu_get_cpus: Get the list of CPUs in the cluster.
*/

Suzuki